/* ************************************************************************
 * Copyright (C) 2020-2023 Advanced Micro Devices, Inc.
 * ************************************************************************ */

#ifndef HIPSOLVER_VERSION_H
#define HIPSOLVER_VERSION_H

/* the configured version and settings
 */
// clang-format off
#define hipsolverVersionMajor 3
#define hipsolverVersionMinor 2
#define hipsolverVersionPatch 0
#define hipsolverVersionTweak dabb6df2b98
// clang-format on

#endif /* HIPSOLVER_VERSION_H */
